#!/usr/bin/env python
#-*-coding: utf-8 -*-
from Tkinter import *
from ttk import Combobox
import ConfigParser
import cv2
from PIL import Image, ImageTk
import time, datetime
import sys,os
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import ftplib
import numpy as np

# **********************************************************************
#
#           MAGNETOMETRE 2.5.1
#
#		La base de donnée SQLite a été remplacée par un système de 
#       fichiers txt.
#
#       Bug du 9 mars 2016: confusion nom fichiers après minuit (ligne
#       301 à ligne 333
#
# **********************************************************************
class MagnetometreGUI(Frame):

	MESURER = False
	NUM_CAMERA = 0
	DELTA_T = 100
	DUREE_INTEG = 5
	RGB_MIN = cv2.cv.Scalar(200,0,0,0)
	RGB_MAX = cv2.cv.Scalar(250,200,200,0)
	CENTROID_X0 = 320
	# type de graphe update
	TOUT = 1
	DERNIER_POINT = 2
	# chemins pour FTP
	LOCAL_DIR = "/graphes"
	LOCAL_DATA_DIR = "datas"
	FTP_DIR = "/observatoire/graphes"
	FTP_LOG_ID = "idFtp"
	FTP_PSWD = "pswdFtp"
	FTP_URL ="ftp:://server.ftp"

# **********************************************************************
	def __init__(self, parent):
		Frame.__init__(self, parent)
		self.parent = parent
		self.parent.geometry("1024x656+30+30")
		self.initGUI()
		self.chargerPreferences()
		self.chargerCamera()
		self.initialiserModele()
		self.poolingMesure()

# **********************************************************************
	def initialiserModele(self):
		# paramètres pour la recherche de la position moyenne du spot
		self.centroid_xInt = 0
		self.centroid_yInt = 0
		self.centroid_x = 0.0
		self.centroid_7 = 0.0
		self.t0 = 0.0
		self.listePoints = []
		self.x_new = None
		self.y_new = None
		self.listePointsGraphique = []
		self.zoom = self.zoomGraphe.get()

		# calcul de la date du jour à 0h00min
		t = time.gmtime(time.time())
		self.t_debut_jour = (datetime.datetime(t.tm_year,t.tm_mon,t.tm_mday,0,0,0) - datetime.datetime(1970,1,1)).total_seconds()
		print "début du jour:",self.t_debut_jour
		# date de publication du graphique
		self.t_publication = time.time()
		print "date publication:",self.t_publication

# **********************************************************************
	def initGUI(self):
		self.bindEvent()
		self.parent.title("Magnétomètre 2.5")
		self.pack(fill=BOTH, expand=1)
		self.graphique =  Canvas(self)
		self.graphique.config(width=960, height=320, bg='black')
		self.graphique.place(x=32,y=32)

		self.cameraVue =  Label(self)
		self.cameraVue.config(bg='black')
		self.cameraVue.place(x=32,y=384)

		self.okButton = Button(self, text="START/STOP",height=3,width=20,
		command=self.startStop)
		self.okButton.place(x=424, y=384)

		self.labelEchelle = Label(self, text="Calcul de l'échelle :",
		font = "Verdana 10 bold")
		self.labelEchelle.place(x=424,y=450)

		self.labelL = Label(self, text="L (en cm)")
		self.labelL.place(x=424,y=482)
		self.entryL = Entry(self, width=15)
		self.entryL.place(x=500, y=482)

		self.labelD = Label(self, text="D (en cm)")
		self.labelD.place(x=424,y=514)
		self.entryDcm = Entry(self, width=15)
		self.entryDcm.place(x=500, y=514)

		self.labelD = Label(self, text="D (en px)")
		self.labelD.place(x=424,y=546)
		self.entryDpx = Entry(self, width=15)
		self.entryDpx.place(x=500, y=546)

		self.labelEchelle = Label(self, text="Publication via FTP :",
		font = "Verdana 10 bold")
		self.labelEchelle.place(x=650,y=450)

		self.checkPubliFTPVar = IntVar()
		self.checkPublication = Checkbutton(self,text = "on/off",variable=self.checkPubliFTPVar,onvalue = 1, offvalue = 0)
		self.checkPublication.place(x=850,y=450)

		self.labelFreqFTP = Label(self, text="Fréq.")
		self.labelFreqFTP.place(x=830,y=546)

		self.listeFreqFTPvaleur = StringVar()
		self.listeFreqFTP = Combobox(self,
		textvariable=self.listeFreqFTPvaleur,state='readonly',width=8)
		self.listeFreqFTP['values'] = ('01 min','05 min', '10 min', '15 min','30 min','45 min','60 min')
		self.listeFreqFTP.current(0)
		self.listeFreqFTP.place(x=875,y=546)

		self.zoomGraphe = Spinbox(self,width=8,from_=0.05,to=5.0,increment=0.05,repeatdelay=100,state="readonly", command=self.echelle)
		self.zoomGraphe.place(x=650,y=384)

		self.zeroButton = Button(self, text="Zéro",height=3,width=6,
		command=self.setZero)
		self.zeroButton.place(x=915, y=384)

# **********************************************************************
	def echelle(self):
		self.zoom = self.zoomGraphe.get()
		self.updateGraphe(self.TOUT)

# **********************************************************************
	def startStop(self):
		self.MESURER = not self.MESURER
		if self.MESURER:
			self.updateGraphe(self.TOUT)
			self.okButton.configure(bg = 'red')
			self.okButton.configure(activebackground = 'red')
		else:
			self.okButton.configure(bg = 'lightgray')
			self.okButton.configure(activebackground = 'lightgray')

# **********************************************************************
	def setZero(self):
		self.CENTROID_X0 = self.centroid_x

# **********************************************************************
	def sauverMesure(self,temps,deltaB_H):
		# il faut aussi updater le graphique ...
		print "... mesure ...", deltaB_H,'nT  ',temps,'s'

		nomFichier =  time.strftime("%Y-%m-%d",time.gmtime())+'.txt'
		chemin = self.LOCAL_DATA_DIR
		
		if not os.path.exists(os.path.dirname("/"+chemin)):
			os.makedirs(os.path.dirname("/"+chemin))
		
		with open(chemin+"/"+nomFichier,'a') as f:
			f.write(str(temps)+' '+str(deltaB_H)+'\n')

		self.listePointsGraphique.append([temps,deltaB_H])
		self.updateGraphe(self.DERNIER_POINT)

# **********************************************************************
	def updateGraphe(self, typeUpdate):
		if typeUpdate == self.TOUT :
			self.graphique.create_rectangle(0, 0, 960, 320, fill='black')
			self.graphique.create_rectangle(44, 55, 944, 275, outline='white')
			self.graphique.create_line(40,165,948,165, fill='white')
			self.graphique.create_line(45,110,943,110, fill='gray')
			self.graphique.create_line(45,220,943,220, fill='gray')

			for i in range(0,25):
				self.graphique.create_line(44+int(i*900/24),56,44+int(i*900/24),274, fill="gray")
				self.graphique.create_text(44+int(i*900/24),274+20, text= i,fill='white')
			self.graphique.create_text(22,110, text= 'Est',fill='white')
			self.graphique.create_text(22,165, text= '0.0',fill='white')
			self.graphique.create_text(22,220, text= 'Ouest',fill='white')

			s = float(self.zoomGraphe.get())
			self.graphique.create_text(22,55, text= "{:3.1f}".format(110/s),fill='white')
			self.graphique.create_text(22,275, text= "{:3.1f}".format(-110/s),fill='white')

			date = time.gmtime(self.t_debut_jour)
			titre = "delta B Horiz. (nT) - "+str(date.tm_year)+" / "+str(date.tm_mon)+" / "+str(date.tm_mday)
			self.graphique.create_text(44,30,anchor='nw', text=titre ,fill='white')

			for (t,B_H) in self.listePointsGraphique:
				self.tracerPoint((t,B_H))

		if typeUpdate == self.DERNIER_POINT:
			(t,B_H) = self.listePointsGraphique[len(self.listePointsGraphique)-1]
			if (t - self.t_debut_jour)> 86400:
				self.t_debut_jour = t
				del self.listePointsGraphique[:]
				self.listePointsGraphique.append([t,B_H])
				print "------------- NOUVELLE JOURNEE"
				self.updateGraphe(self.TOUT)
			else:
				self.tracerPoint((t,B_H))

# **********************************************************************
	def tracerPoint(self, (t,B_H)):
		date = time.gmtime(t)
		heure   = date.tm_hour
		minute  = date.tm_min
		seconde = date.tm_sec
		x = 0.0
		y = 0.0
		s = float(self.zoom)
		x = 44 + heure*900/24+minute*900/(24*60)+ seconde*900/(24*60*60)
		y = 165 - int( s * B_H)
		self.graphique.create_oval(int(x),int(y), int(x),int(y),fill='yellow',outline="yellow")

# **********************************************************************
	def chargerCamera(self):
		print "initialiser caméra ..."
		self.camera = cv2.VideoCapture(self.NUM_CAMERA)
		self.camera.set(cv2.cv.CV_CAP_PROP_FRAME_WIDTH, 640)
		self.camera.set(cv2.cv.CV_CAP_PROP_FRAME_HEIGHT, 480)

# **********************************************************************
	def fermerCamera(self):
		print "fermer caméra ..."
		self.camera.release()

# **********************************************************************
#           BOUCLE PRINCIPALE
# **********************************************************************
	def poolingMesure(self):
	   # capture d'une image
	   self.ret, self.frame = self.camera.read()
	   self.cv2RgbImage = cv2.cvtColor(self.frame, cv2.COLOR_BGR2RGB)
	   # tracking spot laser
	   self.graycv2Img =  cv2.cvtColor(self.frame, cv2.COLOR_BGR2GRAY)
	   ret,thresh = cv2.threshold(self.graycv2Img,200,255,cv2.THRESH_BINARY)
	   moments = cv2.moments(thresh)
	   if moments:
		   if moments['m00']>0:
			   self.centroid_x = moments['m10']/moments['m00']
			   self.centroid_y = moments['m01']/moments['m00']
			   self.centroid_xInt = int(self.centroid_x)
			   self.centroid_yInt = int(self.centroid_y)

	   # dessin du réticule
	   p1 = (0,self.centroid_yInt)
	   p2 = (640,self.centroid_yInt)
	   p3 = ( self.centroid_xInt,0)
	   p4 = ( self.centroid_xInt,480)
	   cv2.line(self.cv2RgbImage,p3,p4,(255,255,0),3)
	   cv2.line(self.cv2RgbImage,p1,p2,(255,255,0),3)

	   # affichage de l'image
	   self.resized = cv2.resize(self.cv2RgbImage,(360,240),interpolation = cv2.INTER_AREA)
	   self.img = Image.fromarray(self.resized)
	   imgTk = ImageTk.PhotoImage(image = self.img)
	   self.imgTk = imgTk
	   self.cameraVue.config(image = imgTk)

	   # mesurer la position moyenne du spot
	   self.t = time.time()
	   self.listePoints.append([self.t , self.centroid_x])
	   if ( self.t - self.t0 > self.DUREE_INTEG ):
		   if self.MESURER:
			   t_m = 0.0
			   x_m = 0.0
			   for (t,x) in self.listePoints:
				   t_m = t_m + t
				   x_m = x_m + x
			   x_m = x_m / len(self.listePoints) - self.CENTROID_X0 #centre écran = origine
			   t_m = t_m / len(self.listePoints)
			   # calculer le champ magnétique
			   self.L = 100.0
			   self.Dcm = 1.0
			   self.Dpx = 1.0
			   try:
				   self.L = float(self.entryL.get())
				   self.Dcm = float(self.entryDcm.get())
				   self.Dpx = float(self.entryDpx.get())
			   except ValueError, e:
				   print e
			   deltaB_H = 0.5 * 21042.0 * self.Dcm / (self.Dpx * self.L ) *(-x_m)
			   self.sauverMesure(t_m,deltaB_H)
		   self.t0 = self.t
		   del self.listePoints[:]

	   #  publication d'un graphique
	   if self.t > (self.t_publication + int(self.listeFreqFTP.get()[:2])*60):
		   self.t_publication = self.t
		   name = self.publicationGraphique()
		   if self.checkPubliFTPVar.get():
			   self.publicationFTP(name)
		   #fin publication
		   print "*** *** *** fin d'un cycle de publication"
	   #  purge des images inutiles
	   del(self.frame)
	   del(self.cv2RgbImage)
	   del(self.graycv2Img)
	   del(thresh)




	   #  recommencer toutes les DELTA_T millisecondes
	   self.parent.after(self.DELTA_T, self.poolingMesure)

# **********************************************************************
	def publicationGraphique(self):
		date = time.gmtime(self.t_debut_jour)
		name = time.strftime("%Y%m%d.png", date)		
		print "**********************  PUBLICATION GRAPHE ",name,"   ",self.t_debut_jour
#		nomFichier =  time.strftime("%Y-%m-%d",time.gmtime())+'.txt'   BUG
		nomFichier =  time.strftime("%Y-%m-%d",date)+'.txt'		# correction
		chemin = self.LOCAL_DATA_DIR
		rows = np.loadtxt(chemin+"/"+nomFichier) 
		print "**********************  PUBLICATION GRAPHE  fin lecture fichier données ..."		
		xmin = float(time.gmtime(self.t_debut_jour).tm_yday)
		xmax = xmin +1
		if len(rows)>0:
			xtemp = []
			ytemp = []
			for row in rows:	
				t = time.gmtime(row[0])
				d = float(t.tm_yday)
				h = float(t.tm_hour)
				m = float(t.tm_min)
				s = float(t.tm_sec)
				d_deci =  d + (h + (m + s/60.0)/60.0)/24.0
				xtemp.append(d_deci)
				ytemp.append(row[1])
			print "**********************  PUBLICATION GRAPHE  début tracer courbe ..."
			fig = plt.figure(figsize=(15.0,4.10))
			plt.grid(True)
			plt.plot(xtemp,ytemp,'.',color=(1,0.647,0),markersize=3)
			plt.ylim(-160,160)
			plt.xlim(xmin - 0.01,xmax+0.01)
			plt.xticks(np.linspace(xmin,xmax,13))
			plt.xlabel('jour (GMT)')
			plt.ylabel(r'$\Delta B_H$ (nT)')
			plt.tight_layout()
			print "**********************  PUBLICATION GRAPHE  sauver la courbe ..."
			fig.savefig(self.LOCAL_DIR + "/"+ name,dpi=72, format="png")
			plt.close()
		print "**********************  PUBLICATION GRAPHE terminée ..."
		del(rows)
		return name

# **********************************************************************
	def publicationFTP(self, name):
		print "**********************  PUBLICATION FTP de ", name," ..."
		try:
			serveur = ftplib.FTP(self.FTP_URL,self.FTP_LOG_ID,self.FTP_PSWD,None,5)
			#serveur.login(self.FTP_LOG_ID,self.FTP_PSWD)
			serveur.cwd(self.FTP_DIR)
			fichier = self.LOCAL_DIR+"/"+name
			file = open(fichier, 'rb')
			serveur.storbinary('STOR '+ name ,file)
			serveur.close()
			print "**********************  PUBLICATION FTP de ", name,"   OK"
		#except Exception, e:
		except Exception, e:
			#print str(e)
			#print "**********************  PUBLICATION FTP: I/O error({0}): {1}".format(e.errno, e.strerror)
			print "**********************  PUBLICATION FTP: Erreur ... "

# **********************************************************************
	def chargerPreferences(self):
		self.preferences = ConfigParser.RawConfigParser()
		self.preferences.read('preferences.cfg')
		try:
			self.entryL.insert(0,self.preferences.get('ECHELLE','L (cm)'))
			self.entryDcm.insert(0,self.preferences.get('ECHELLE','D (cm)'))
			self.entryDpx.insert(0,self.preferences.get('ECHELLE','D (px)'))
			self.CENTROID_X0 = self.preferences.getfloat('ECHELLE','CENTROID_X0 (px)')
			
			self.LOCAL_DATA_DIR = self.preferences.get('LOCAL','LOCAL_DATA_DIR')

			self.LOCAL_DIR = self.preferences.get('TRANSFERT_FTP','LOCAL_DIR')
			self.FTP_DIR = self.preferences.get('TRANSFERT_FTP','FTP_DIR')
			self.FTP_LOG_ID = self.preferences.get('TRANSFERT_FTP','USER')
			self.FTP_PSWD = self.preferences.get('TRANSFERT_FTP','PASSWORD')
			self.FTP_URL = self.preferences.get('TRANSFERT_FTP','URL')
			self.listeFreqFTP.current(self.preferences.getint('TRANSFERT_FTP','FREQ. (min)'))

			self.DUREE_INTEG = int(self.preferences.get('CAMERA','DUREE_INTEG'))
			self.NUM_CAMERA = self.preferences.getint('CAMERA','NUM_CAMERA')

		except ConfigParser.Error, e:
			print str(e)

# **********************************************************************
	def sauverPreferences(self):
		self.preferences = ConfigParser.RawConfigParser()
		self.preferences.add_section('ECHELLE')
		self.preferences.set('ECHELLE','L (cm)',self.entryL.get())
		self.preferences.set('ECHELLE','D (cm)',self.entryDcm.get())
		self.preferences.set('ECHELLE','D (px)',self.entryDpx.get())
		self.preferences.set('ECHELLE','CENTROID_X0 (px)',self.CENTROID_X0)
		
		self.preferences.add_section('LOCAL')
		self.preferences.set('LOCAL','LOCAL_DATA_DIR',self.LOCAL_DATA_DIR)
		
		self.preferences.add_section('TRANSFERT_FTP')
		self.preferences.set('TRANSFERT_FTP','URL',self.FTP_URL)
		self.preferences.set('TRANSFERT_FTP','USER',self.FTP_LOG_ID)
		self.preferences.set('TRANSFERT_FTP','PASSWORD',self.FTP_PSWD )
		self.preferences.set('TRANSFERT_FTP','FREQ. (min)',self.listeFreqFTP.current())
		self.preferences.set('TRANSFERT_FTP','LOCAL_DIR',self.LOCAL_DIR )
		self.preferences.set('TRANSFERT_FTP','FTP_DIR',self.FTP_DIR )


		self.preferences.add_section('CAMERA')
		self.preferences.set('CAMERA','NUM_CAMERA',self.NUM_CAMERA)
		self.preferences.set('CAMERA','DUREE_INTEG',self.DUREE_INTEG)

		with open('preferences.cfg', 'wb') as configfile:
			self.preferences.write(configfile)

# **********************************************************************
	def fermetureAppli(self):
		self.sauverPreferences()
		self.fermerCamera()
		self.parent.destroy()

# **********************************************************************
	def bindEvent(self):
		self.parent.protocol("WM_DELETE_WINDOW", self.fermetureAppli)

# **********************************************************************
#
#       MAIN
#
# **********************************************************************
if __name__ == '__main__':
	root = Tk()

	app = MagnetometreGUI(root)
	root.mainloop()

